import { Input, Component } from 'angular-ts-decorators';
import { CorTableColumn } from './cor-table-col.component';

/**
 * A component that displays a table of information, with optional filtering and automatic sorting.
 */
@Component({
  selector: 'corTable',
  templateUrl: '/static/js/directives/ui/cor-table/cor-table.component.html',
  transclude: true,
})
export class CorTableComponent implements ng.IComponentController {
  @Input('=') public tableData: any[];
  @Input('@') public tableItemTitle: string;
  @Input('<') public filterFields: string[];
  @Input('@') public compact: string;
  @Input('<') public maxDisplayCount: number;

  private columns: CorTableColumn[];
  private orderedData: any;
  private options: any;

  constructor(private TableService: any) {
    this.columns = [];
    this.options = {
      'filter': '',
      'reverse': false,
      'predicate': '',
      'page': 0,
    };
  }

  public addColumn(col: CorTableColumn): void {
    this.columns.push(col);

    if (col.selected == 'true') {
      this.options['predicate'] = col.datafield;
    }

    this.refreshOrder();
  }

  private setOrder(col: CorTableColumn): void {
    this.TableService.orderBy(col.datafield, this.options);
    this.refreshOrder();
  }

  private tablePredicateClass(col: CorTableColumn, options: any) {
    return this.TableService.tablePredicateClass(col.datafield, this.options.predicate,
                                                 this.options.reverse);
  }

  private refreshOrder(): void {
    var columnMap = {};
    this.columns.forEach(function(col) {
      columnMap[col.datafield] = col;
    });

    var filterCols = this.columns.filter(function(col) {
      return !!col.sortfield;
    }).map((col) => (col.datafield));

    var numericCols = this.columns.filter(function(col) {
      return col.isNumeric();
    }).map((col) => (col.datafield));

    var processed = this.tableData.map((item) => {
      var keys = Object.keys(item);
      var newObj = {};
      for (var i = 0; i < keys.length; ++i) {
        var key = keys[i];
        if (columnMap[key]) {
          newObj[key] = columnMap[key].processColumnForOrdered(this.TableService, item[key]);
        }
      }
      return newObj;
    });

    this.orderedData = this.TableService.buildOrderedItems(processed, this.options,
            filterCols, numericCols);
  }
}
